from dataclasses import dataclass

from ....base import BaseMessageWrapper
from .....proto.cosmos.bank.v1beta1 import MsgMultiSend as MsgMultiSendProto
from .....proto.cosmos.bank.v1beta1 import MsgSend as MsgSendProto
from .....proto.cosmos.base import v1beta1 as __base_v1_beta1__
from .....proto.cosmos.base.query import v1beta1 as __base_query_v1_beta1__

assert __base_v1_beta1__
assert __base_query_v1_beta1__


class MsgSend(BaseMessageWrapper, MsgSendProto):
    @property
    def type_url(self):
        return "/cosmos.bank.v1beta1.MsgSend"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgSend"


class MsgMultiSend(BaseMessageWrapper, MsgMultiSendProto):
    @property
    def type_url(self):
        return "/cosmos.bank.v1beta1.MsgMultiSend"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgMultiSend"



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/reflection/v1beta1/reflection.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class ListAllInterfacesRequest(betterproto.Message):
    """
    ListAllInterfacesRequest is the request type of the ListAllInterfaces RPC.
    """

    pass


@dataclass(eq=False, repr=False)
class ListAllInterfacesResponse(betterproto.Message):
    """
    ListAllInterfacesResponse is the response type of the ListAllInterfaces
    RPC.
    """

    interface_names: List[str] = betterproto.string_field(1)
    """interface_names is an array of all the registered interfaces."""


@dataclass(eq=False, repr=False)
class ListImplementationsRequest(betterproto.Message):
    """
    ListImplementationsRequest is the request type of the ListImplementations
    RPC.
    """

    interface_name: str = betterproto.string_field(1)
    """
    interface_name defines the interface to query the implementations for.
    """


@dataclass(eq=False, repr=False)
class ListImplementationsResponse(betterproto.Message):
    """
    ListImplementationsResponse is the response type of the ListImplementations
    RPC.
    """

    implementation_message_names: List[str] = betterproto.string_field(1)


class ReflectionServiceStub(betterproto.ServiceStub):
    async def list_all_interfaces(
        self,
        list_all_interfaces_request: "ListAllInterfacesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ListAllInterfacesResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v1beta1.ReflectionService/ListAllInterfaces",
            list_all_interfaces_request,
            ListAllInterfacesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_implementations(
        self,
        list_implementations_request: "ListImplementationsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ListImplementationsResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v1beta1.ReflectionService/ListImplementations",
            list_implementations_request,
            ListImplementationsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ReflectionServiceBase(ServiceBase):
    async def list_all_interfaces(
        self, list_all_interfaces_request: "ListAllInterfacesRequest"
    ) -> "ListAllInterfacesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_implementations(
        self, list_implementations_request: "ListImplementationsRequest"
    ) -> "ListImplementationsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list_all_interfaces(
        self,
        stream: "grpclib.server.Stream[ListAllInterfacesRequest, ListAllInterfacesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_all_interfaces(request)
        await stream.send_message(response)

    async def __rpc_list_implementations(
        self,
        stream: "grpclib.server.Stream[ListImplementationsRequest, ListImplementationsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_implementations(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.base.reflection.v1beta1.ReflectionService/ListAllInterfaces": grpclib.const.Handler(
                self.__rpc_list_all_interfaces,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListAllInterfacesRequest,
                ListAllInterfacesResponse,
            ),
            "/cosmos.base.reflection.v1beta1.ReflectionService/ListImplementations": grpclib.const.Handler(
                self.__rpc_list_implementations,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListImplementationsRequest,
                ListImplementationsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crypto/multisig/keys.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class LegacyAminoPubKey(betterproto.Message):
    """
    LegacyAminoPubKey specifies a public key type which nests multiple public
    keys and a threshold, it uses legacy amino address rules.
    """

    threshold: int = betterproto.uint32_field(1)
    public_keys: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crypto/multisig/v1beta1/multisig.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class MultiSignature(betterproto.Message):
    """
    MultiSignature wraps the signatures from a multisig.LegacyAminoPubKey. See
    cosmos.tx.v1betata1.ModeInfo.Multi for how to specify which signers signed
    and with which modes.
    """

    signatures: List[bytes] = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class CompactBitArray(betterproto.Message):
    """
    CompactBitArray is an implementation of a space efficient bit array. This
    is used to ensure that the encoded data takes up a minimal amount of space
    after proto encoding. This is not thread safe, and is not intended for
    concurrent usage.
    """

    extra_bits_stored: int = betterproto.uint32_field(1)
    elems: bytes = betterproto.bytes_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/distribution/v1beta1/distribution.proto, cosmos/distribution/v1beta1/genesis.proto, cosmos/distribution/v1beta1/query.proto, cosmos/distribution/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base import v1beta1 as __base_v1_beta1__
from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgSetWithdrawAddress(betterproto.Message):
    """
    MsgSetWithdrawAddress sets the withdraw address for a delegator (or
    validator self-delegation).
    """

    delegator_address: str = betterproto.string_field(1)
    withdraw_address: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class MsgSetWithdrawAddressResponse(betterproto.Message):
    """
    MsgSetWithdrawAddressResponse defines the Msg/SetWithdrawAddress response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgWithdrawDelegatorReward(betterproto.Message):
    """
    MsgWithdrawDelegatorReward represents delegation withdrawal to a delegator
    from a single validator.
    """

    delegator_address: str = betterproto.string_field(1)
    validator_address: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class MsgWithdrawDelegatorRewardResponse(betterproto.Message):
    """
    MsgWithdrawDelegatorRewardResponse defines the Msg/WithdrawDelegatorReward
    response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgWithdrawValidatorCommission(betterproto.Message):
    """
    MsgWithdrawValidatorCommission withdraws the full commission to the
    validator address.
    """

    validator_address: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class MsgWithdrawValidatorCommissionResponse(betterproto.Message):
    """
    MsgWithdrawValidatorCommissionResponse defines the
    Msg/WithdrawValidatorCommission response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgFundCommunityPool(betterproto.Message):
    """
    MsgFundCommunityPool allows an account to directly fund the community pool.
    """

    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(1)
    depositor: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class MsgFundCommunityPoolResponse(betterproto.Message):
    """
    MsgFundCommunityPoolResponse defines the Msg/FundCommunityPool response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the set of params for the distribution module."""

    community_tax: str = betterproto.string_field(1)
    base_proposer_reward: str = betterproto.string_field(2)
    bonus_proposer_reward: str = betterproto.string_field(3)
    withdraw_addr_enabled: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class ValidatorHistoricalRewards(betterproto.Message):
    """
    ValidatorHistoricalRewards represents historical rewards for a validator.
    Height is implicit within the store key. Cumulative reward ratio is the sum
    from the zeroeth period until this period of rewards / tokens, per the
    spec. The reference count indicates the number of objects which might need
    to reference this historical entry at any point. ReferenceCount =    number
    of outstanding delegations which ended the associated period (and    might
    need to read that record)  + number of slashes which ended the associated
    period (and might need to  read that record)  + one per validator for the
    zeroeth period, set on initialization
    """

    cumulative_reward_ratio: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)
    reference_count: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ValidatorCurrentRewards(betterproto.Message):
    """
    ValidatorCurrentRewards represents current rewards and current period for a
    validator kept as a running counter and incremented each block as long as
    the validator's tokens remain constant.
    """

    rewards: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)
    period: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ValidatorAccumulatedCommission(betterproto.Message):
    """
    ValidatorAccumulatedCommission represents accumulated commission for a
    validator kept as a running counter, can be withdrawn at any time.
    """

    commission: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ValidatorOutstandingRewards(betterproto.Message):
    """
    ValidatorOutstandingRewards represents outstanding (un-withdrawn) rewards
    for a validator inexpensive to track, allows simple sanity checks.
    """

    rewards: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ValidatorSlashEvent(betterproto.Message):
    """
    ValidatorSlashEvent represents a validator slash event. Height is implicit
    within the store key. This is needed to calculate appropriate amount of
    staking tokens for delegations which are withdrawn after a slash has
    occurred.
    """

    validator_period: int = betterproto.uint64_field(1)
    fraction: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ValidatorSlashEvents(betterproto.Message):
    """
    ValidatorSlashEvents is a collection of ValidatorSlashEvent messages.
    """

    validator_slash_events: List["ValidatorSlashEvent"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class FeePool(betterproto.Message):
    """FeePool is the global fee pool for distribution."""

    community_pool: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CommunityPoolSpendProposal(betterproto.Message):
    """
    CommunityPoolSpendProposal details a proposal for use of community funds,
    together with how many coins are proposed to be spent, and to which
    recipient account.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    recipient: str = betterproto.string_field(3)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class DelegatorStartingInfo(betterproto.Message):
    """
    DelegatorStartingInfo represents the starting info for a delegator reward
    period. It tracks the previous validator period, the delegation's amount of
    staking token, and the creation height (to check later on if any slashes
    have occurred). NOTE: Even though validators are slashed to whole staking
    tokens, the delegators within the validator may be left with less than a
    full token, thus sdk.Dec is used.
    """

    previous_period: int = betterproto.uint64_field(1)
    stake: str = betterproto.string_field(2)
    height: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class DelegationDelegatorReward(betterproto.Message):
    """
    DelegationDelegatorReward represents the properties of a delegator's
    delegation reward.
    """

    validator_address: str = betterproto.string_field(1)
    reward: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CommunityPoolSpendProposalWithDeposit(betterproto.Message):
    """
    CommunityPoolSpendProposalWithDeposit defines a CommunityPoolSpendProposal
    with a deposit
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    recipient: str = betterproto.string_field(3)
    amount: str = betterproto.string_field(4)
    deposit: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: "Params" = betterproto.message_field(1)
    """params defines the parameters of the module."""


@dataclass(eq=False, repr=False)
class QueryValidatorOutstandingRewardsRequest(betterproto.Message):
    """
    QueryValidatorOutstandingRewardsRequest is the request type for the
    Query/ValidatorOutstandingRewards RPC method.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address defines the validator address to query for."""


@dataclass(eq=False, repr=False)
class QueryValidatorOutstandingRewardsResponse(betterproto.Message):
    """
    QueryValidatorOutstandingRewardsResponse is the response type for the
    Query/ValidatorOutstandingRewards RPC method.
    """

    rewards: "ValidatorOutstandingRewards" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryValidatorCommissionRequest(betterproto.Message):
    """
    QueryValidatorCommissionRequest is the request type for the
    Query/ValidatorCommission RPC method
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address defines the validator address to query for."""


@dataclass(eq=False, repr=False)
class QueryValidatorCommissionResponse(betterproto.Message):
    """
    QueryValidatorCommissionResponse is the response type for the
    Query/ValidatorCommission RPC method
    """

    commission: "ValidatorAccumulatedCommission" = betterproto.message_field(1)
    """commission defines the commision the validator received."""


@dataclass(eq=False, repr=False)
class QueryValidatorSlashesRequest(betterproto.Message):
    """
    QueryValidatorSlashesRequest is the request type for the
    Query/ValidatorSlashes RPC method
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address defines the validator address to query for."""

    starting_height: int = betterproto.uint64_field(2)
    """
    starting_height defines the optional starting height to query the slashes.
    """

    ending_height: int = betterproto.uint64_field(3)
    """
    starting_height defines the optional ending height to query the slashes.
    """

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(4)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryValidatorSlashesResponse(betterproto.Message):
    """
    QueryValidatorSlashesResponse is the response type for the
    Query/ValidatorSlashes RPC method.
    """

    slashes: List["ValidatorSlashEvent"] = betterproto.message_field(1)
    """slashes defines the slashes the validator received."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryDelegationRewardsRequest(betterproto.Message):
    """
    QueryDelegationRewardsRequest is the request type for the
    Query/DelegationRewards RPC method.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address defines the delegator address to query for."""

    validator_address: str = betterproto.string_field(2)
    """validator_address defines the validator address to query for."""


@dataclass(eq=False, repr=False)
class QueryDelegationRewardsResponse(betterproto.Message):
    """
    QueryDelegationRewardsResponse is the response type for the
    Query/DelegationRewards RPC method.
    """

    rewards: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)
    """rewards defines the rewards accrued by a delegation."""


@dataclass(eq=False, repr=False)
class QueryDelegationTotalRewardsRequest(betterproto.Message):
    """
    QueryDelegationTotalRewardsRequest is the request type for the
    Query/DelegationTotalRewards RPC method.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address defines the delegator address to query for."""


@dataclass(eq=False, repr=False)
class QueryDelegationTotalRewardsResponse(betterproto.Message):
    """
    QueryDelegationTotalRewardsResponse is the response type for the
    Query/DelegationTotalRewards RPC method.
    """

    rewards: List["DelegationDelegatorReward"] = betterproto.message_field(1)
    """rewards defines all the rewards accrued by a delegator."""

    total: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(2)
    """total defines the sum of all the rewards."""


@dataclass(eq=False, repr=False)
class QueryDelegatorValidatorsRequest(betterproto.Message):
    """
    QueryDelegatorValidatorsRequest is the request type for the
    Query/DelegatorValidators RPC method.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address defines the delegator address to query for."""


@dataclass(eq=False, repr=False)
class QueryDelegatorValidatorsResponse(betterproto.Message):
    """
    QueryDelegatorValidatorsResponse is the response type for the
    Query/DelegatorValidators RPC method.
    """

    validators: List[str] = betterproto.string_field(1)
    """validators defines the validators a delegator is delegating for."""


@dataclass(eq=False, repr=False)
class QueryDelegatorWithdrawAddressRequest(betterproto.Message):
    """
    QueryDelegatorWithdrawAddressRequest is the request type for the
    Query/DelegatorWithdrawAddress RPC method.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address defines the delegator address to query for."""


@dataclass(eq=False, repr=False)
class QueryDelegatorWithdrawAddressResponse(betterproto.Message):
    """
    QueryDelegatorWithdrawAddressResponse is the response type for the
    Query/DelegatorWithdrawAddress RPC method.
    """

    withdraw_address: str = betterproto.string_field(1)
    """withdraw_address defines the delegator address to query for."""


@dataclass(eq=False, repr=False)
class QueryCommunityPoolRequest(betterproto.Message):
    """
    QueryCommunityPoolRequest is the request type for the Query/CommunityPool
    RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryCommunityPoolResponse(betterproto.Message):
    """
    QueryCommunityPoolResponse is the response type for the Query/CommunityPool
    RPC method.
    """

    pool: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(1)
    """pool defines community pool's coins."""


@dataclass(eq=False, repr=False)
class DelegatorWithdrawInfo(betterproto.Message):
    """
    DelegatorWithdrawInfo is the address for where distributions rewards are
    withdrawn to by default this struct is only used at genesis to feed in
    default withdraw addresses.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address is the address of the delegator."""

    withdraw_address: str = betterproto.string_field(2)
    """
    withdraw_address is the address to withdraw the delegation rewards to.
    """


@dataclass(eq=False, repr=False)
class ValidatorOutstandingRewardsRecord(betterproto.Message):
    """
    ValidatorOutstandingRewardsRecord is used for import/export via genesis
    json.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address is the address of the validator."""

    outstanding_rewards: List["__base_v1_beta1__.DecCoin"] = betterproto.message_field(2)
    """
    outstanding_rewards represents the oustanding rewards of a validator.
    """


@dataclass(eq=False, repr=False)
class ValidatorAccumulatedCommissionRecord(betterproto.Message):
    """
    ValidatorAccumulatedCommissionRecord is used for import / export via
    genesis json.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address is the address of the validator."""

    accumulated: "ValidatorAccumulatedCommission" = betterproto.message_field(2)
    """accumulated is the accumulated commission of a validator."""


@dataclass(eq=False, repr=False)
class ValidatorHistoricalRewardsRecord(betterproto.Message):
    """
    ValidatorHistoricalRewardsRecord is used for import / export via genesis
    json.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address is the address of the validator."""

    period: int = betterproto.uint64_field(2)
    """period defines the period the historical rewards apply to."""

    rewards: "ValidatorHistoricalRewards" = betterproto.message_field(3)
    """rewards defines the historical rewards of a validator."""


@dataclass(eq=False, repr=False)
class ValidatorCurrentRewardsRecord(betterproto.Message):
    """
    ValidatorCurrentRewardsRecord is used for import / export via genesis json.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address is the address of the validator."""

    rewards: "ValidatorCurrentRewards" = betterproto.message_field(2)
    """rewards defines the current rewards of a validator."""


@dataclass(eq=False, repr=False)
class DelegatorStartingInfoRecord(betterproto.Message):
    """
    DelegatorStartingInfoRecord used for import / export via genesis json.
    """

    delegator_address: str = betterproto.string_field(1)
    """delegator_address is the address of the delegator."""

    validator_address: str = betterproto.string_field(2)
    """validator_address is the address of the validator."""

    starting_info: "DelegatorStartingInfo" = betterproto.message_field(3)
    """starting_info defines the starting info of a delegator."""


@dataclass(eq=False, repr=False)
class ValidatorSlashEventRecord(betterproto.Message):
    """
    ValidatorSlashEventRecord is used for import / export via genesis json.
    """

    validator_address: str = betterproto.string_field(1)
    """validator_address is the address of the validator."""

    height: int = betterproto.uint64_field(2)
    """height defines the block height at which the slash event occured."""

    period: int = betterproto.uint64_field(3)
    """period is the period of the slash event."""

    validator_slash_event: "ValidatorSlashEvent" = betterproto.message_field(4)
    """validator_slash_event describes the slash event."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the distribution module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    """params defines all the paramaters of the module."""

    fee_pool: "FeePool" = betterproto.message_field(2)
    """fee_pool defines the fee pool at genesis."""

    delegator_withdraw_infos: List["DelegatorWithdrawInfo"] = betterproto.message_field(3)
    """fee_pool defines the delegator withdraw infos at genesis."""

    previous_proposer: str = betterproto.string_field(4)
    """fee_pool defines the previous proposer at genesis."""

    outstanding_rewards: List["ValidatorOutstandingRewardsRecord"] = betterproto.message_field(5)
    """
    fee_pool defines the outstanding rewards of all validators at genesis.
    """

    validator_accumulated_commissions: List["ValidatorAccumulatedCommissionRecord"] = betterproto.message_field(6)
    """
    fee_pool defines the accumulated commisions of all validators at genesis.
    """

    validator_historical_rewards: List["ValidatorHistoricalRewardsRecord"] = betterproto.message_field(7)
    """
    fee_pool defines the historical rewards of all validators at genesis.
    """

    validator_current_rewards: List["ValidatorCurrentRewardsRecord"] = betterproto.message_field(8)
    """fee_pool defines the current rewards of all validators at genesis."""

    delegator_starting_infos: List["DelegatorStartingInfoRecord"] = betterproto.message_field(9)
    """fee_pool defines the delegator starting infos at genesis."""

    validator_slash_events: List["ValidatorSlashEventRecord"] = betterproto.message_field(10)
    """fee_pool defines the validator slash events at genesis."""


class MsgStub(betterproto.ServiceStub):
    async def set_withdraw_address(
        self,
        msg_set_withdraw_address: "MsgSetWithdrawAddress",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgSetWithdrawAddressResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Msg/SetWithdrawAddress",
            msg_set_withdraw_address,
            MsgSetWithdrawAddressResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def withdraw_delegator_reward(
        self,
        msg_withdraw_delegator_reward: "MsgWithdrawDelegatorReward",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgWithdrawDelegatorRewardResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Msg/WithdrawDelegatorReward",
            msg_withdraw_delegator_reward,
            MsgWithdrawDelegatorRewardResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def withdraw_validator_commission(
        self,
        msg_withdraw_validator_commission: "MsgWithdrawValidatorCommission",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgWithdrawValidatorCommissionResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Msg/WithdrawValidatorCommission",
            msg_withdraw_validator_commission,
            MsgWithdrawValidatorCommissionResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def fund_community_pool(
        self,
        msg_fund_community_pool: "MsgFundCommunityPool",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgFundCommunityPoolResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Msg/FundCommunityPool",
            msg_fund_community_pool,
            MsgFundCommunityPoolResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def validator_outstanding_rewards(
        self,
        query_validator_outstanding_rewards_request: "QueryValidatorOutstandingRewardsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryValidatorOutstandingRewardsResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/ValidatorOutstandingRewards",
            query_validator_outstanding_rewards_request,
            QueryValidatorOutstandingRewardsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def validator_commission(
        self,
        query_validator_commission_request: "QueryValidatorCommissionRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryValidatorCommissionResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/ValidatorCommission",
            query_validator_commission_request,
            QueryValidatorCommissionResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def validator_slashes(
        self,
        query_validator_slashes_request: "QueryValidatorSlashesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryValidatorSlashesResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/ValidatorSlashes",
            query_validator_slashes_request,
            QueryValidatorSlashesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delegation_rewards(
        self,
        query_delegation_rewards_request: "QueryDelegationRewardsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDelegationRewardsResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/DelegationRewards",
            query_delegation_rewards_request,
            QueryDelegationRewardsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delegation_total_rewards(
        self,
        query_delegation_total_rewards_request: "QueryDelegationTotalRewardsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDelegationTotalRewardsResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/DelegationTotalRewards",
            query_delegation_total_rewards_request,
            QueryDelegationTotalRewardsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delegator_validators(
        self,
        query_delegator_validators_request: "QueryDelegatorValidatorsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDelegatorValidatorsResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/DelegatorValidators",
            query_delegator_validators_request,
            QueryDelegatorValidatorsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def delegator_withdraw_address(
        self,
        query_delegator_withdraw_address_request: "QueryDelegatorWithdrawAddressRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDelegatorWithdrawAddressResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/DelegatorWithdrawAddress",
            query_delegator_withdraw_address_request,
            QueryDelegatorWithdrawAddressResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def community_pool(
        self,
        query_community_pool_request: "QueryCommunityPoolRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryCommunityPoolResponse":
        return await self._unary_unary(
            "/cosmos.distribution.v1beta1.Query/CommunityPool",
            query_community_pool_request,
            QueryCommunityPoolResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def set_withdraw_address(
        self, msg_set_withdraw_address: "MsgSetWithdrawAddress"
    ) -> "MsgSetWithdrawAddressResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def withdraw_delegator_reward(
        self, msg_withdraw_delegator_reward: "MsgWithdrawDelegatorReward"
    ) -> "MsgWithdrawDelegatorRewardResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def withdraw_validator_commission(
        self, msg_withdraw_validator_commission: "MsgWithdrawValidatorCommission"
    ) -> "MsgWithdrawValidatorCommissionResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def fund_community_pool(
        self, msg_fund_community_pool: "MsgFundCommunityPool"
    ) -> "MsgFundCommunityPoolResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_set_withdraw_address(
        self,
        stream: "grpclib.server.Stream[MsgSetWithdrawAddress, MsgSetWithdrawAddressResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.set_withdraw_address(request)
        await stream.send_message(response)

    async def __rpc_withdraw_delegator_reward(
        self,
        stream: "grpclib.server.Stream[MsgWithdrawDelegatorReward, MsgWithdrawDelegatorRewardResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.withdraw_delegator_reward(request)
        await stream.send_message(response)

    async def __rpc_withdraw_validator_commission(
        self,
        stream: "grpclib.server.Stream[MsgWithdrawValidatorCommission, MsgWithdrawValidatorCommissionResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.withdraw_validator_commission(request)
        await stream.send_message(response)

    async def __rpc_fund_community_pool(
        self,
        stream: "grpclib.server.Stream[MsgFundCommunityPool, MsgFundCommunityPoolResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.fund_community_pool(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.distribution.v1beta1.Msg/SetWithdrawAddress": grpclib.const.Handler(
                self.__rpc_set_withdraw_address,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgSetWithdrawAddress,
                MsgSetWithdrawAddressResponse,
            ),
            "/cosmos.distribution.v1beta1.Msg/WithdrawDelegatorReward": grpclib.const.Handler(
                self.__rpc_withdraw_delegator_reward,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgWithdrawDelegatorReward,
                MsgWithdrawDelegatorRewardResponse,
            ),
            "/cosmos.distribution.v1beta1.Msg/WithdrawValidatorCommission": grpclib.const.Handler(
                self.__rpc_withdraw_validator_commission,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgWithdrawValidatorCommission,
                MsgWithdrawValidatorCommissionResponse,
            ),
            "/cosmos.distribution.v1beta1.Msg/FundCommunityPool": grpclib.const.Handler(
                self.__rpc_fund_community_pool,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgFundCommunityPool,
                MsgFundCommunityPoolResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validator_outstanding_rewards(
        self,
        query_validator_outstanding_rewards_request: "QueryValidatorOutstandingRewardsRequest",
    ) -> "QueryValidatorOutstandingRewardsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validator_commission(
        self, query_validator_commission_request: "QueryValidatorCommissionRequest"
    ) -> "QueryValidatorCommissionResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validator_slashes(
        self, query_validator_slashes_request: "QueryValidatorSlashesRequest"
    ) -> "QueryValidatorSlashesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delegation_rewards(
        self, query_delegation_rewards_request: "QueryDelegationRewardsRequest"
    ) -> "QueryDelegationRewardsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delegation_total_rewards(
        self,
        query_delegation_total_rewards_request: "QueryDelegationTotalRewardsRequest",
    ) -> "QueryDelegationTotalRewardsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delegator_validators(
        self, query_delegator_validators_request: "QueryDelegatorValidatorsRequest"
    ) -> "QueryDelegatorValidatorsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delegator_withdraw_address(
        self,
        query_delegator_withdraw_address_request: "QueryDelegatorWithdrawAddressRequest",
    ) -> "QueryDelegatorWithdrawAddressResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def community_pool(
        self, query_community_pool_request: "QueryCommunityPoolRequest"
    ) -> "QueryCommunityPoolResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_validator_outstanding_rewards(
        self,
        stream: "grpclib.server.Stream[QueryValidatorOutstandingRewardsRequest, QueryValidatorOutstandingRewardsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.validator_outstanding_rewards(request)
        await stream.send_message(response)

    async def __rpc_validator_commission(
        self,
        stream: "grpclib.server.Stream[QueryValidatorCommissionRequest, QueryValidatorCommissionResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.validator_commission(request)
        await stream.send_message(response)

    async def __rpc_validator_slashes(
        self,
        stream: "grpclib.server.Stream[QueryValidatorSlashesRequest, QueryValidatorSlashesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.validator_slashes(request)
        await stream.send_message(response)

    async def __rpc_delegation_rewards(
        self,
        stream: "grpclib.server.Stream[QueryDelegationRewardsRequest, QueryDelegationRewardsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delegation_rewards(request)
        await stream.send_message(response)

    async def __rpc_delegation_total_rewards(
        self,
        stream: "grpclib.server.Stream[QueryDelegationTotalRewardsRequest, QueryDelegationTotalRewardsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delegation_total_rewards(request)
        await stream.send_message(response)

    async def __rpc_delegator_validators(
        self,
        stream: "grpclib.server.Stream[QueryDelegatorValidatorsRequest, QueryDelegatorValidatorsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delegator_validators(request)
        await stream.send_message(response)

    async def __rpc_delegator_withdraw_address(
        self,
        stream: "grpclib.server.Stream[QueryDelegatorWithdrawAddressRequest, QueryDelegatorWithdrawAddressResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.delegator_withdraw_address(request)
        await stream.send_message(response)

    async def __rpc_community_pool(
        self,
        stream: "grpclib.server.Stream[QueryCommunityPoolRequest, QueryCommunityPoolResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.community_pool(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.distribution.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/ValidatorOutstandingRewards": grpclib.const.Handler(
                self.__rpc_validator_outstanding_rewards,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryValidatorOutstandingRewardsRequest,
                QueryValidatorOutstandingRewardsResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/ValidatorCommission": grpclib.const.Handler(
                self.__rpc_validator_commission,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryValidatorCommissionRequest,
                QueryValidatorCommissionResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/ValidatorSlashes": grpclib.const.Handler(
                self.__rpc_validator_slashes,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryValidatorSlashesRequest,
                QueryValidatorSlashesResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/DelegationRewards": grpclib.const.Handler(
                self.__rpc_delegation_rewards,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDelegationRewardsRequest,
                QueryDelegationRewardsResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/DelegationTotalRewards": grpclib.const.Handler(
                self.__rpc_delegation_total_rewards,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDelegationTotalRewardsRequest,
                QueryDelegationTotalRewardsResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/DelegatorValidators": grpclib.const.Handler(
                self.__rpc_delegator_validators,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDelegatorValidatorsRequest,
                QueryDelegatorValidatorsResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/DelegatorWithdrawAddress": grpclib.const.Handler(
                self.__rpc_delegator_withdraw_address,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDelegatorWithdrawAddressRequest,
                QueryDelegatorWithdrawAddressResponse,
            ),
            "/cosmos.distribution.v1beta1.Query/CommunityPool": grpclib.const.Handler(
                self.__rpc_community_pool,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCommunityPoolRequest,
                QueryCommunityPoolResponse,
            ),
        }



import json
from typing import List, Tuple, Iterable

from betterproto.lib.google.protobuf import Any as AnyProto

from .client import Client
from .constants import MAX_MEMO_CHARACTERS
from .exceptions import EmptyMsgError, UndefinedError, ValueTooLargeError
from .messages.base import BaseMessageWrapper
from .proto.cosmos.base.v1beta1 import Coin
from .proto.cosmos.tx.signing.v1beta1 import SignMode
from .proto.cosmos.tx.v1beta1 import Fee, TxBody, ModeInfo, SignerInfo, AuthInfo, SignDoc, TxRaw, ModeInfoSingle
from .wallet.public_key import PublicKey


class Transaction:
    def __init__(
        self,
        msgs: List[BaseMessageWrapper] = None,
        account_num: int = None,
        sequence: int = None,
        chain_id: str = None,
        fee: List[Coin] = None,
        gas: int = 200000,
        memo: str = "",
    ):
        self.msgs = msgs if msgs is not None else []
        self.account_num = account_num
        self.sequence = sequence
        self.chain_id = chain_id
        self.fee = Fee(amount=fee, gas_limit=gas)
        self.gas = gas
        self.memo = memo

    @staticmethod
    def __convert_msgs(msgs: Iterable[BaseMessageWrapper]) -> List[AnyProto]:
        return [AnyProto(type_url=msg.type_url, value=bytes(msg)) for msg in msgs]

    def with_messages(self, *msgs: BaseMessageWrapper) -> "Transaction":
        self.msgs.extend(msgs)
        return self

    async def with_sender(self, client: Client, sender: str) -> "Transaction":
        if len(self.msgs) == 0:
            raise EmptyMsgError("message is empty, please use with_messages at least 1 message")

        account = await client.get_account(sender)
        self.account_num = account.account_number
        self.sequence = account.sequence
        return self

    def with_account_num(self, account_num: int) -> "Transaction":
        self.account_num = account_num
        return self

    def with_sequence(self, sequence: int) -> "Transaction":
        self.sequence = sequence
        return self

    def with_chain_id(self, chain_id: str) -> "Transaction":
        self.chain_id = chain_id
        return self

    def with_fee(self, fee: List[Coin]) -> "Transaction":
        self.fee = Fee(amount=fee, gas_limit=self.fee.gas_limit)
        return self

    def with_gas(self, gas: int) -> "Transaction":
        """Sets Transaction's gas.

        Args:
            gas: Gas

        Returns:
            A Transaction instance.
        """

        self.fee.gas_limit = gas
        return self

    def with_memo(self, memo: str) -> "Transaction":
        """Sets Transaction's memo.

        Args:
            memo: Memo with a maximum length of 256.

        Returns:
            A Transaction instance.
        """

        if len(memo) > MAX_MEMO_CHARACTERS:
            raise ValueTooLargeError("memo is too large")
        self.memo = memo
        return self

    def __generate_info(self, public_key: PublicKey, sign_mode: SignMode) -> Tuple[bytes, bytes]:
        body = TxBody(
            messages=self.__convert_msgs(self.msgs),
            memo=self.memo,
        )

        mode_info = ModeInfo(ModeInfoSingle(mode=sign_mode))
        if public_key is not None:
            pub_key_proto = public_key.to_public_key_proto()
            any_public_key = AnyProto(type_url="/cosmos.crypto.secp256k1.PubKey", value=bytes(pub_key_proto))
            signer_info = SignerInfo(mode_info=mode_info, sequence=self.sequence, public_key=any_public_key)
        else:
            signer_info = SignerInfo(mode_info=mode_info, sequence=self.sequence)

        auth_info = AuthInfo(signer_infos=[signer_info], fee=self.fee)

        return bytes(body), bytes(auth_info)

    def get_sign_doc(self, public_key: PublicKey = None) -> SignDoc:
        """Returns the sign data from Transaction.

        Args:
            public_key: A PublicKey instance.

        Returns:
            A SignDoc instance containing the transaction detail.
        """

        if len(self.msgs) == 0:
            raise EmptyMsgError("message is empty")

        if self.account_num is None:
            raise UndefinedError("account_num should be defined")

        if self.sequence is None:
            raise UndefinedError("sequence should be defined")

        if self.chain_id is None:
            raise UndefinedError("chain_id should be defined")

        body_bytes, auth_info_bytes = self.__generate_info(public_key, SignMode.SIGN_MODE_DIRECT)

        return SignDoc(
            body_bytes=body_bytes,
            auth_info_bytes=auth_info_bytes,
            chain_id=self.chain_id,
            account_number=self.account_num,
        )

    def get_sign_message_for_legacy_codec(self) -> bytes:
        """Returns the transaction encoded in Cosmos's legacy format.

        Returns:
            Legacy transaction as byte.
        """

        msg = {
            "account_number": str(self.account_num),
            "chain_id": self.chain_id,
            "fee": {
                "amount": [fee_amount.to_dict() for fee_amount in self.fee.amount],
                "gas": str(self.fee.gas_limit),
            },
            "memo": self.memo,
            "msgs": [msg.to_legacy_codec() for msg in self.msgs],
            "sequence": str(self.sequence),
        }
        return json.dumps(msg, separators=(",", ":"), sort_keys=True).encode("utf-8")

    def get_tx_data(
        self, signature: bytes, public_key: PublicKey = None, sign_mode: SignMode = SignMode.SIGN_MODE_DIRECT
    ) -> bytes:
        """Returns the transaction as a byte.

        Args:
            signature: Signature from get_sign_doc().
            public_key: A public key instance.
            sign_mode: SignMode.

        Returns:
            Transaction as byte.
        """

        body_bytes, auth_info_bytes = self.__generate_info(public_key, sign_mode)
        tx_raw = TxRaw(body_bytes=body_bytes, auth_info_bytes=auth_info_bytes, signatures=[signature])
        return bytes(tx_raw)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crypto/ed25519/keys.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class PubKey(betterproto.Message):
    """
    PubKey is an ed25519 public key for handling Tendermint keys in SDK. It's
    needed for Any serialization and SDK compatibility. It must not be used in
    a non Tendermint key context because it doesn't implement ADR-28.
    Nevertheless, you will like to use ed25519 in app user level then you must
    create a new proto message and follow ADR-28 for Address construction.
    """

    key: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class PrivKey(betterproto.Message):
    """
    Deprecated: PrivKey defines a ed25519 private key. NOTE: ed25519 keys must
    not be used in SDK apps except in a tendermint validator context.
    """

    key: bytes = betterproto.bytes_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: tendermint/abci/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    crypto as _crypto__,
    types as _types__,
)


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class CheckTxType(betterproto.Enum):
    NEW = 0
    RECHECK = 1


class EvidenceType(betterproto.Enum):
    UNKNOWN = 0
    DUPLICATE_VOTE = 1
    LIGHT_CLIENT_ATTACK = 2


class ResponseOfferSnapshotResult(betterproto.Enum):
    UNKNOWN = 0
    ACCEPT = 1
    ABORT = 2
    REJECT = 3
    REJECT_FORMAT = 4
    REJECT_SENDER = 5


class ResponseApplySnapshotChunkResult(betterproto.Enum):
    UNKNOWN = 0
    ACCEPT = 1
    ABORT = 2
    RETRY = 3
    RETRY_SNAPSHOT = 4
    REJECT_SNAPSHOT = 5


@dataclass(eq=False, repr=False)
class Request(betterproto.Message):
    echo: "RequestEcho" = betterproto.message_field(1, group="value")
    flush: "RequestFlush" = betterproto.message_field(2, group="value")
    info: "RequestInfo" = betterproto.message_field(3, group="value")
    set_option: "RequestSetOption" = betterproto.message_field(4, group="value")
    init_chain: "RequestInitChain" = betterproto.message_field(5, group="value")
    query: "RequestQuery" = betterproto.message_field(6, group="value")
    begin_block: "RequestBeginBlock" = betterproto.message_field(7, group="value")
    check_tx: "RequestCheckTx" = betterproto.message_field(8, group="value")
    deliver_tx: "RequestDeliverTx" = betterproto.message_field(9, group="value")
    end_block: "RequestEndBlock" = betterproto.message_field(10, group="value")
    commit: "RequestCommit" = betterproto.message_field(11, group="value")
    list_snapshots: "RequestListSnapshots" = betterproto.message_field(12, group="value")
    offer_snapshot: "RequestOfferSnapshot" = betterproto.message_field(13, group="value")
    load_snapshot_chunk: "RequestLoadSnapshotChunk" = betterproto.message_field(14, group="value")
    apply_snapshot_chunk: "RequestApplySnapshotChunk" = betterproto.message_field(15, group="value")


@dataclass(eq=False, repr=False)
class RequestEcho(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class RequestFlush(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class RequestInfo(betterproto.Message):
    version: str = betterproto.string_field(1)
    block_version: int = betterproto.uint64_field(2)
    p2_p_version: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class RequestSetOption(betterproto.Message):
    """nondeterministic"""

    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RequestInitChain(betterproto.Message):
    time: datetime = betterproto.message_field(1)
    chain_id: str = betterproto.string_field(2)
    consensus_params: "ConsensusParams" = betterproto.message_field(3)
    validators: List["ValidatorUpdate"] = betterproto.message_field(4)
    app_state_bytes: bytes = betterproto.bytes_field(5)
    initial_height: int = betterproto.int64_field(6)


@dataclass(eq=False, repr=False)
class RequestQuery(betterproto.Message):
    data: bytes = betterproto.bytes_field(1)
    path: str = betterproto.string_field(2)
    height: int = betterproto.int64_field(3)
    prove: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class RequestBeginBlock(betterproto.Message):
    hash: bytes = betterproto.bytes_field(1)
    header: "_types__.Header" = betterproto.message_field(2)
    last_commit_info: "LastCommitInfo" = betterproto.message_field(3)
    byzantine_validators: List["Evidence"] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class RequestCheckTx(betterproto.Message):
    tx: bytes = betterproto.bytes_field(1)
    type: "CheckTxType" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class RequestDeliverTx(betterproto.Message):
    tx: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class RequestEndBlock(betterproto.Message):
    height: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class RequestCommit(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class RequestListSnapshots(betterproto.Message):
    """lists available snapshots"""

    pass


@dataclass(eq=False, repr=False)
class RequestOfferSnapshot(betterproto.Message):
    """offers a snapshot to the application"""

    snapshot: "Snapshot" = betterproto.message_field(1)
    app_hash: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class RequestLoadSnapshotChunk(betterproto.Message):
    """loads a snapshot chunk"""

    height: int = betterproto.uint64_field(1)
    format: int = betterproto.uint32_field(2)
    chunk: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class RequestApplySnapshotChunk(betterproto.Message):
    """Applies a snapshot chunk"""

    index: int = betterproto.uint32_field(1)
    chunk: bytes = betterproto.bytes_field(2)
    sender: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Response(betterproto.Message):
    exception: "ResponseException" = betterproto.message_field(1, group="value")
    echo: "ResponseEcho" = betterproto.message_field(2, group="value")
    flush: "ResponseFlush" = betterproto.message_field(3, group="value")
    info: "ResponseInfo" = betterproto.message_field(4, group="value")
    set_option: "ResponseSetOption" = betterproto.message_field(5, group="value")
    init_chain: "ResponseInitChain" = betterproto.message_field(6, group="value")
    query: "ResponseQuery" = betterproto.message_field(7, group="value")
    begin_block: "ResponseBeginBlock" = betterproto.message_field(8, group="value")
    check_tx: "ResponseCheckTx" = betterproto.message_field(9, group="value")
    deliver_tx: "ResponseDeliverTx" = betterproto.message_field(10, group="value")
    end_block: "ResponseEndBlock" = betterproto.message_field(11, group="value")
    commit: "ResponseCommit" = betterproto.message_field(12, group="value")
    list_snapshots: "ResponseListSnapshots" = betterproto.message_field(13, group="value")
    offer_snapshot: "ResponseOfferSnapshot" = betterproto.message_field(14, group="value")
    load_snapshot_chunk: "ResponseLoadSnapshotChunk" = betterproto.message_field(15, group="value")
    apply_snapshot_chunk: "ResponseApplySnapshotChunk" = betterproto.message_field(16, group="value")


@dataclass(eq=False, repr=False)
class ResponseException(betterproto.Message):
    """nondeterministic"""

    error: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ResponseEcho(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ResponseFlush(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ResponseInfo(betterproto.Message):
    data: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    app_version: int = betterproto.uint64_field(3)
    last_block_height: int = betterproto.int64_field(4)
    last_block_app_hash: bytes = betterproto.bytes_field(5)


@dataclass(eq=False, repr=False)
class ResponseSetOption(betterproto.Message):
    """nondeterministic"""

    code: int = betterproto.uint32_field(1)
    log: str = betterproto.string_field(3)
    """bytes data = 2;"""

    info: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class ResponseInitChain(betterproto.Message):
    consensus_params: "ConsensusParams" = betterproto.message_field(1)
    validators: List["ValidatorUpdate"] = betterproto.message_field(2)
    app_hash: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ResponseQuery(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    log: str = betterproto.string_field(3)
    """bytes data = 2; // use "value" instead."""

    info: str = betterproto.string_field(4)
    index: int = betterproto.int64_field(5)
    key: bytes = betterproto.bytes_field(6)
    value: bytes = betterproto.bytes_field(7)
    proof_ops: "_crypto__.ProofOps" = betterproto.message_field(8)
    height: int = betterproto.int64_field(9)
    codespace: str = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class ResponseBeginBlock(betterproto.Message):
    events: List["Event"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ResponseCheckTx(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)
    log: str = betterproto.string_field(3)
    info: str = betterproto.string_field(4)
    gas_wanted: int = betterproto.int64_field(5)
    gas_used: int = betterproto.int64_field(6)
    events: List["Event"] = betterproto.message_field(7)
    codespace: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class ResponseDeliverTx(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)
    log: str = betterproto.string_field(3)
    info: str = betterproto.string_field(4)
    gas_wanted: int = betterproto.int64_field(5)
    gas_used: int = betterproto.int64_field(6)
    events: List["Event"] = betterproto.message_field(7)
    codespace: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class ResponseEndBlock(betterproto.Message):
    validator_updates: List["ValidatorUpdate"] = betterproto.message_field(1)
    consensus_param_updates: "ConsensusParams" = betterproto.message_field(2)
    events: List["Event"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ResponseCommit(betterproto.Message):
    data: bytes = betterproto.bytes_field(2)
    """reserve 1"""

    retain_height: int = betterproto.int64_field(3)


@dataclass(eq=False, repr=False)
class ResponseListSnapshots(betterproto.Message):
    snapshots: List["Snapshot"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ResponseOfferSnapshot(betterproto.Message):
    result: "ResponseOfferSnapshotResult" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ResponseLoadSnapshotChunk(betterproto.Message):
    chunk: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class ResponseApplySnapshotChunk(betterproto.Message):
    result: "ResponseApplySnapshotChunkResult" = betterproto.enum_field(1)
    refetch_chunks: List[int] = betterproto.uint32_field(2)
    reject_senders: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ConsensusParams(betterproto.Message):
    """
    ConsensusParams contains all consensus-relevant parameters that can be
    adjusted by the abci app
    """

    block: "BlockParams" = betterproto.message_field(1)
    evidence: "_types__.EvidenceParams" = betterproto.message_field(2)
    validator: "_types__.ValidatorParams" = betterproto.message_field(3)
    version: "_types__.VersionParams" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class BlockParams(betterproto.Message):
    """BlockParams contains limits on the block size."""

    max_bytes: int = betterproto.int64_field(1)
    """Note: must be greater than 0"""

    max_gas: int = betterproto.int64_field(2)
    """Note: must be greater or equal to -1"""


@dataclass(eq=False, repr=False)
class LastCommitInfo(betterproto.Message):
    round: int = betterproto.int32_field(1)
    votes: List["VoteInfo"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """
    Event allows application developers to attach additional information to
    ResponseBeginBlock, ResponseEndBlock, ResponseCheckTx and
    ResponseDeliverTx. Later, transactions may be queried using these events.
    """

    type: str = betterproto.string_field(1)
    attributes: List["EventAttribute"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class EventAttribute(betterproto.Message):
    """EventAttribute is a single key-value pair, associated with an event."""

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    index: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class TxResult(betterproto.Message):
    """
    TxResult contains results of executing the transaction. One usage is
    indexing transaction results.
    """

    height: int = betterproto.int64_field(1)
    index: int = betterproto.uint32_field(2)
    tx: bytes = betterproto.bytes_field(3)
    result: "ResponseDeliverTx" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class Validator(betterproto.Message):
    """Validator"""

    address: bytes = betterproto.bytes_field(1)
    power: int = betterproto.int64_field(3)
    """PubKey pub_key = 2 [(gogoproto.nullable)=false];"""


@dataclass(eq=False, repr=False)
class ValidatorUpdate(betterproto.Message):
    """ValidatorUpdate"""

    pub_key: "_crypto__.PublicKey" = betterproto.message_field(1)
    power: int = betterproto.int64_field(2)


@dataclass(eq=False, repr=False)
class VoteInfo(betterproto.Message):
    """VoteInfo"""

    validator: "Validator" = betterproto.message_field(1)
    signed_last_block: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class Evidence(betterproto.Message):
    type: "EvidenceType" = betterproto.enum_field(1)
    validator: "Validator" = betterproto.message_field(2)
    """The offending validator"""

    height: int = betterproto.int64_field(3)
    """The height when the offense occurred"""

    time: datetime = betterproto.message_field(4)
    """The corresponding time where the offense occurred"""

    total_voting_power: int = betterproto.int64_field(5)
    """
    Total voting power of the validator set in case the ABCI application does
    not store historical validators.
    https://github.com/tendermint/tendermint/issues/4581
    """


@dataclass(eq=False, repr=False)
class Snapshot(betterproto.Message):
    height: int = betterproto.uint64_field(1)
    format: int = betterproto.uint32_field(2)
    chunks: int = betterproto.uint32_field(3)
    hash: bytes = betterproto.bytes_field(4)
    metadata: bytes = betterproto.bytes_field(5)


class AbciApplicationStub(betterproto.ServiceStub):
    async def echo(
        self,
        request_echo: "RequestEcho",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseEcho":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/Echo",
            request_echo,
            ResponseEcho,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def flush(
        self,
        request_flush: "RequestFlush",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseFlush":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/Flush",
            request_flush,
            ResponseFlush,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def info(
        self,
        request_info: "RequestInfo",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseInfo":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/Info",
            request_info,
            ResponseInfo,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def set_option(
        self,
        request_set_option: "RequestSetOption",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseSetOption":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/SetOption",
            request_set_option,
            ResponseSetOption,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deliver_tx(
        self,
        request_deliver_tx: "RequestDeliverTx",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseDeliverTx":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/DeliverTx",
            request_deliver_tx,
            ResponseDeliverTx,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def check_tx(
        self,
        request_check_tx: "RequestCheckTx",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseCheckTx":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/CheckTx",
            request_check_tx,
            ResponseCheckTx,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query(
        self,
        request_query: "RequestQuery",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseQuery":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/Query",
            request_query,
            ResponseQuery,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def commit(
        self,
        request_commit: "RequestCommit",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseCommit":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/Commit",
            request_commit,
            ResponseCommit,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def init_chain(
        self,
        request_init_chain: "RequestInitChain",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseInitChain":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/InitChain",
            request_init_chain,
            ResponseInitChain,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def begin_block(
        self,
        request_begin_block: "RequestBeginBlock",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseBeginBlock":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/BeginBlock",
            request_begin_block,
            ResponseBeginBlock,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def end_block(
        self,
        request_end_block: "RequestEndBlock",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseEndBlock":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/EndBlock",
            request_end_block,
            ResponseEndBlock,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_snapshots(
        self,
        request_list_snapshots: "RequestListSnapshots",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseListSnapshots":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/ListSnapshots",
            request_list_snapshots,
            ResponseListSnapshots,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def offer_snapshot(
        self,
        request_offer_snapshot: "RequestOfferSnapshot",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseOfferSnapshot":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/OfferSnapshot",
            request_offer_snapshot,
            ResponseOfferSnapshot,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def load_snapshot_chunk(
        self,
        request_load_snapshot_chunk: "RequestLoadSnapshotChunk",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseLoadSnapshotChunk":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/LoadSnapshotChunk",
            request_load_snapshot_chunk,
            ResponseLoadSnapshotChunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def apply_snapshot_chunk(
        self,
        request_apply_snapshot_chunk: "RequestApplySnapshotChunk",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ResponseApplySnapshotChunk":
        return await self._unary_unary(
            "/tendermint.abci.ABCIApplication/ApplySnapshotChunk",
            request_apply_snapshot_chunk,
            ResponseApplySnapshotChunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class AbciApplicationBase(ServiceBase):
    async def echo(self, request_echo: "RequestEcho") -> "ResponseEcho":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def flush(self, request_flush: "RequestFlush") -> "ResponseFlush":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def info(self, request_info: "RequestInfo") -> "ResponseInfo":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_option(self, request_set_option: "RequestSetOption") -> "ResponseSetOption":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deliver_tx(self, request_deliver_tx: "RequestDeliverTx") -> "ResponseDeliverTx":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def check_tx(self, request_check_tx: "RequestCheckTx") -> "ResponseCheckTx":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query(self, request_query: "RequestQuery") -> "ResponseQuery":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def commit(self, request_commit: "RequestCommit") -> "ResponseCommit":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def init_chain(self, request_init_chain: "RequestInitChain") -> "ResponseInitChain":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def begin_block(self, request_begin_block: "RequestBeginBlock") -> "ResponseBeginBlock":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def end_block(self, request_end_block: "RequestEndBlock") -> "ResponseEndBlock":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_snapshots(self, request_list_snapshots: "RequestListSnapshots") -> "ResponseListSnapshots":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def offer_snapshot(self, request_offer_snapshot: "RequestOfferSnapshot") -> "ResponseOfferSnapshot":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def load_snapshot_chunk(
        self, request_load_snapshot_chunk: "RequestLoadSnapshotChunk"
    ) -> "ResponseLoadSnapshotChunk":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def apply_snapshot_chunk(
        self, request_apply_snapshot_chunk: "RequestApplySnapshotChunk"
    ) -> "ResponseApplySnapshotChunk":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_echo(self, stream: "grpclib.server.Stream[RequestEcho, ResponseEcho]") -> None:
        request = await stream.recv_message()
        response = await self.echo(request)
        await stream.send_message(response)

    async def __rpc_flush(self, stream: "grpclib.server.Stream[RequestFlush, ResponseFlush]") -> None:
        request = await stream.recv_message()
        response = await self.flush(request)
        await stream.send_message(response)

    async def __rpc_info(self, stream: "grpclib.server.Stream[RequestInfo, ResponseInfo]") -> None:
        request = await stream.recv_message()
        response = await self.info(request)
        await stream.send_message(response)

    async def __rpc_set_option(self, stream: "grpclib.server.Stream[RequestSetOption, ResponseSetOption]") -> None:
        request = await stream.recv_message()
        response = await self.set_option(request)
        await stream.send_message(response)

    async def __rpc_deliver_tx(self, stream: "grpclib.server.Stream[RequestDeliverTx, ResponseDeliverTx]") -> None:
        request = await stream.recv_message()
        response = await self.deliver_tx(request)
        await stream.send_message(response)

    async def __rpc_check_tx(self, stream: "grpclib.server.Stream[RequestCheckTx, ResponseCheckTx]") -> None:
        request = await stream.recv_message()
        response = await self.check_tx(request)
        await stream.send_message(response)

    async def __rpc_query(self, stream: "grpclib.server.Stream[RequestQuery, ResponseQuery]") -> None:
        request = await stream.recv_message()
        response = await self.query(request)
        await stream.send_message(response)

    async def __rpc_commit(self, stream: "grpclib.server.Stream[RequestCommit, ResponseCommit]") -> None:
        request = await stream.recv_message()
        response = await self.commit(request)
        await stream.send_message(response)

    async def __rpc_init_chain(self, stream: "grpclib.server.Stream[RequestInitChain, ResponseInitChain]") -> None:
        request = await stream.recv_message()
        response = await self.init_chain(request)
        await stream.send_message(response)

    async def __rpc_begin_block(self, stream: "grpclib.server.Stream[RequestBeginBlock, ResponseBeginBlock]") -> None:
        request = await stream.recv_message()
        response = await self.begin_block(request)
        await stream.send_message(response)

    async def __rpc_end_block(self, stream: "grpclib.server.Stream[RequestEndBlock, ResponseEndBlock]") -> None:
        request = await stream.recv_message()
        response = await self.end_block(request)
        await stream.send_message(response)

    async def __rpc_list_snapshots(
        self,
        stream: "grpclib.server.Stream[RequestListSnapshots, ResponseListSnapshots]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_snapshots(request)
        await stream.send_message(response)

    async def __rpc_offer_snapshot(
        self,
        stream: "grpclib.server.Stream[RequestOfferSnapshot, ResponseOfferSnapshot]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.offer_snapshot(request)
        await stream.send_message(response)

    async def __rpc_load_snapshot_chunk(
        self,
        stream: "grpclib.server.Stream[RequestLoadSnapshotChunk, ResponseLoadSnapshotChunk]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.load_snapshot_chunk(request)
        await stream.send_message(response)

    async def __rpc_apply_snapshot_chunk(
        self,
        stream: "grpclib.server.Stream[RequestApplySnapshotChunk, ResponseApplySnapshotChunk]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.apply_snapshot_chunk(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/tendermint.abci.ABCIApplication/Echo": grpclib.const.Handler(
                self.__rpc_echo,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestEcho,
                ResponseEcho,
            ),
            "/tendermint.abci.ABCIApplication/Flush": grpclib.const.Handler(
                self.__rpc_flush,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestFlush,
                ResponseFlush,
            ),
            "/tendermint.abci.ABCIApplication/Info": grpclib.const.Handler(
                self.__rpc_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestInfo,
                ResponseInfo,
            ),
            "/tendermint.abci.ABCIApplication/SetOption": grpclib.const.Handler(
                self.__rpc_set_option,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestSetOption,
                ResponseSetOption,
            ),
            "/tendermint.abci.ABCIApplication/DeliverTx": grpclib.const.Handler(
                self.__rpc_deliver_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestDeliverTx,
                ResponseDeliverTx,
            ),
            "/tendermint.abci.ABCIApplication/CheckTx": grpclib.const.Handler(
                self.__rpc_check_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestCheckTx,
                ResponseCheckTx,
            ),
            "/tendermint.abci.ABCIApplication/Query": grpclib.const.Handler(
                self.__rpc_query,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestQuery,
                ResponseQuery,
            ),
            "/tendermint.abci.ABCIApplication/Commit": grpclib.const.Handler(
                self.__rpc_commit,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestCommit,
                ResponseCommit,
            ),
            "/tendermint.abci.ABCIApplication/InitChain": grpclib.const.Handler(
                self.__rpc_init_chain,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestInitChain,
                ResponseInitChain,
            ),
            "/tendermint.abci.ABCIApplication/BeginBlock": grpclib.const.Handler(
                self.__rpc_begin_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestBeginBlock,
                ResponseBeginBlock,
            ),
            "/tendermint.abci.ABCIApplication/EndBlock": grpclib.const.Handler(
                self.__rpc_end_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestEndBlock,
                ResponseEndBlock,
            ),
            "/tendermint.abci.ABCIApplication/ListSnapshots": grpclib.const.Handler(
                self.__rpc_list_snapshots,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestListSnapshots,
                ResponseListSnapshots,
            ),
            "/tendermint.abci.ABCIApplication/OfferSnapshot": grpclib.const.Handler(
                self.__rpc_offer_snapshot,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestOfferSnapshot,
                ResponseOfferSnapshot,
            ),
            "/tendermint.abci.ABCIApplication/LoadSnapshotChunk": grpclib.const.Handler(
                self.__rpc_load_snapshot_chunk,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestLoadSnapshotChunk,
                ResponseLoadSnapshotChunk,
            ),
            "/tendermint.abci.ABCIApplication/ApplySnapshotChunk": grpclib.const.Handler(
                self.__rpc_apply_snapshot_chunk,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestApplySnapshotChunk,
                ResponseApplySnapshotChunk,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crisis/v1beta1/genesis.proto, cosmos/crisis/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base import v1beta1 as __base_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgVerifyInvariant(betterproto.Message):
    """
    MsgVerifyInvariant represents a message to verify a particular invariance.
    """

    sender: str = betterproto.string_field(1)
    invariant_module_name: str = betterproto.string_field(2)
    invariant_route: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgVerifyInvariantResponse(betterproto.Message):
    """
    MsgVerifyInvariantResponse defines the Msg/VerifyInvariant response type.
    """

    pass


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the crisis module's genesis state."""

    constant_fee: "__base_v1_beta1__.Coin" = betterproto.message_field(3)
    """
    constant_fee is the fee used to verify the invariant in the crisis module.
    """


class MsgStub(betterproto.ServiceStub):
    async def verify_invariant(
        self,
        msg_verify_invariant: "MsgVerifyInvariant",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgVerifyInvariantResponse":
        return await self._unary_unary(
            "/cosmos.crisis.v1beta1.Msg/VerifyInvariant",
            msg_verify_invariant,
            MsgVerifyInvariantResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def verify_invariant(self, msg_verify_invariant: "MsgVerifyInvariant") -> "MsgVerifyInvariantResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_verify_invariant(
        self,
        stream: "grpclib.server.Stream[MsgVerifyInvariant, MsgVerifyInvariantResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.verify_invariant(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.crisis.v1beta1.Msg/VerifyInvariant": grpclib.const.Handler(
                self.__rpc_verify_invariant,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgVerifyInvariant,
                MsgVerifyInvariantResponse,
            ),
        }



import hashlib
from typing import Optional

from bech32 import bech32_decode, convertbits, bech32_encode
from ecdsa.curves import SECP256k1
from ecdsa.keys import VerifyingKey, BadSignatureError

from .address import Address
from .constants import BECH32_PUBKEY_ACC_PREFIX, BECH32_PUBKEY_VAL_PREFIX, BECH32_PUBKEY_CONS_PREFIX
from ..exceptions import DecodeError
from ..proto.cosmos.crypto.secp256k1 import PubKey as PubKeyProto


class PublicKey:
    """Class for wrapping VerifyKey used for signature verification. Contains methods to encode or decode Bech32.

    Attributes:
        verify_key: the ecdsa VerifyingKey instance
    """

    def __init__(self, _error_do_not_use_init_directly=None) -> None:
        """Unsupported, please do not construct it directly."""

        if not _error_do_not_use_init_directly:
            raise TypeError("Please use PublicKey's factory methods to construct me")
        self.verify_key: Optional[VerifyingKey] = None

    @classmethod
    def _from_bech32(cls, bech: str, prefix: str) -> "PublicKey":
        hrp, bz = bech32_decode(bech)
        assert hrp == prefix, "Invalid bech32 prefix"
        if bz is None:
            raise DecodeError("Cannot decode bech32")
        bz = convertbits(bz, 5, 8, False)
        self = cls(_error_do_not_use_init_directly=True)
        self.verify_key = VerifyingKey.from_string(bytes(bz[5:]), curve=SECP256k1, hashfunc=hashlib.sha256)
        return self

    @classmethod
    def from_acc_bech32(cls, bech: str) -> "PublicKey":
        """Creates a PublicKey from a bech32 string with an account public key prefix.

        Args:
            bech: Bech32 string with an account public key prefix.

        Returns:
            A PublicKey instance.
        """

        return cls._from_bech32(bech, BECH32_PUBKEY_ACC_PREFIX)

    @classmethod
    def from_val_bech32(cls, bech: str) -> "PublicKey":
        """Creates a PublicKey from a bech32 string with a validator public key prefix.

        Args:
            bech: Bech32 string with a validator public key prefix.

        Returns:
            A PublicKey instance.
        """

        return cls._from_bech32(bech, BECH32_PUBKEY_VAL_PREFIX)

    @classmethod
    def from_cons_bech32(cls, bech: str) -> "PublicKey":
        """Creates a PublicKey from a bech32 string with a validator consensus public key prefix.

        Args:
            bech: Bech32 string with a validator consensus public key prefix.

        Returns:
            A PublicKey instance.
        """

        return cls._from_bech32(bech, BECH32_PUBKEY_CONS_PREFIX)

    @classmethod
    def from_hex(cls, pub: bytes) -> "PublicKey":
        """Creates a PublicKey from a hex representation of the verifying key.

        Args:
            pub: A hex representation of the verifying key.

        Returns:
            A PublicKey instance.
        """

        self = cls(_error_do_not_use_init_directly=True)
        self.verify_key = VerifyingKey.from_string(pub, curve=SECP256k1, hashfunc=hashlib.sha256)
        return self

    def to_hex(self) -> str:
        """Returns a hex representation of the verifying key.

        Returns:
            A hex representation of verified key.
        """

        return self.verify_key.to_string("compressed").hex()

    def to_public_key_proto(self) -> PubKeyProto:
        """Returns a public key of type protobuf.

        Returns:
            A PubKeyProto instance.
        """

        return PubKeyProto(key=self.verify_key.to_string("compressed"))

    def _to_bech32(self, prefix: str) -> str:
        five_bit_r = convertbits(
            # Append prefix public key type follow amino spec.
            bytes.fromhex("eb5ae98721") + self.verify_key.to_string("compressed"),
            8,
            5,
        )
        assert five_bit_r is not None, "Unsuccessful bech32.convertbits call"
        return bech32_encode(prefix, five_bit_r)

    def to_acc_bech32(self) -> str:
        """Returns a bech32 string with an account public key prefix.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_PUBKEY_ACC_PREFIX)

    def to_val_bech32(self) -> str:
        """Returns a bech32 string with a validator public key prefix.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_PUBKEY_VAL_PREFIX)

    def to_cons_bech32(self) -> str:
        """Returns a bech32 string with a validator consensus public key prefix.

        Returns:
            A bech32 address as a string.
        """

        return self._to_bech32(BECH32_PUBKEY_CONS_PREFIX)

    def to_address(self) -> Address:
        """Return address instance from this public key.

        Returns:
            An Address instance.
        """

        hash = hashlib.new("sha256", self.verify_key.to_string("compressed")).digest()
        return Address(hashlib.new("ripemd160", hash).digest())

    def verify(self, msg: bytes, sig: bytes) -> bool:
        """Verify a signature made from the given message.

        Args:
            msg: Data signed by the signature.
            sig: The encoded signature.

        Raises:
            BadSignatureError: If the signature is invalid or malformed.

        Returns:
            True if the verification was successful, false otherwise.
        """

        try:
            return self.verify_key.verify(sig, msg, hashfunc=hashlib.sha256)
        except BadSignatureError:
            return False



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crypto/secp256k1/keys.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class PubKey(betterproto.Message):
    """
    PubKey defines a secp256k1 public key Key is the compressed form of the
    pubkey. The first byte depends is a 0x02 byte if the y-coordinate is the
    lexicographically largest of the two associated with the x-coordinate.
    Otherwise the first byte is a 0x03. This prefix is followed with the
    x-coordinate.
    """

    key: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class PrivKey(betterproto.Message):
    """PrivKey defines a secp256k1 private key."""

    key: bytes = betterproto.bytes_field(1)



from dataclasses import dataclass


@dataclass
class ReferencePriceUpdated(object):
    base: int
    quote: int


@dataclass
class ReferencePrice(object):
    pair: str
    rate: float
    updated_at: ReferencePriceUpdated



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/api/annotations.proto, google/api/http.proto, google/api/httpbody.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class Http(betterproto.Message):
    """
    Defines the HTTP configuration for an API service. It contains a list of
    [HttpRule][google.api.HttpRule], each specifying the mapping of an RPC
    method to one or more HTTP REST API methods.
    """

    rules: List["HttpRule"] = betterproto.message_field(1)
    """
    A list of HTTP configuration rules that apply to individual API methods.
    **NOTE:** All service configuration rules follow "last one wins" order.
    """

    fully_decode_reserved_expansion: bool = betterproto.bool_field(2)
    """
    When set to true, URL path parmeters will be fully URI-decoded except in
    cases of single segment matches in reserved expansion, where "%2F" will be
    left encoded. The default behavior is to not decode RFC 6570 reserved
    characters in multi segment matches.
    """


@dataclass(eq=False, repr=False)
class HttpRule(betterproto.Message):
    """
    `HttpRule` defines the mapping of an RPC method to one or more HTTP REST
    API methods. The mapping specifies how different portions of the RPC
    request message are mapped to URL path, URL query parameters, and HTTP
    request body. The mapping is typically specified as an `google.api.http`
    annotation on the RPC method, see "google/api/annotations.proto" for
    details. The mapping consists of a field specifying the path template and
    method kind.  The path template can refer to fields in the request message,
    as in the example below which describes a REST GET operation on a resource
    collection of messages:     service Messaging {       rpc
    GetMessage(GetMessageRequest) returns (Message) {         option
    (google.api.http).get = "/v1/messages/{message_id}/{sub.subfield}";       }
    }     message GetMessageRequest {       message SubMessage {         string
    subfield = 1;       }       string message_id = 1; // mapped to the URL
    SubMessage sub = 2;    // `sub.subfield` is url-mapped     }     message
    Message {       string text = 1; // content of the resource     } The same
    http annotation can alternatively be expressed inside the `GRPC API
    Configuration` YAML file.     http:       rules:         - selector:
    <proto_package_name>.Messaging.GetMessage           get:
    /v1/messages/{message_id}/{sub.subfield} This definition enables an
    automatic, bidrectional mapping of HTTP JSON to RPC. Example: HTTP | RPC
    -----|----- `GET /v1/messages/123456/foo`  | `GetMessage(message_id:
    "123456" sub: SubMessage(subfield: "foo"))` In general, not only fields but
    also field paths can be referenced from a path pattern. Fields mapped to
    the path pattern cannot be repeated and must have a primitive (non-message)
    type. Any fields in the request message which are not bound by the path
    pattern automatically become (optional) HTTP query parameters. Assume the
    following definition of the request message:     service Messaging {
    rpc GetMessage(GetMessageRequest) returns (Message) {         option
    (google.api.http).get = "/v1/messages/{message_id}";       }     }
    message GetMessageRequest {       message SubMessage {         string
    subfield = 1;       }       string message_id = 1; // mapped to the URL
    int64 revision = 2;    // becomes a parameter       SubMessage sub = 3;
    // `sub.subfield` becomes a parameter     } This enables a HTTP JSON to RPC
    mapping as below: HTTP | RPC -----|----- `GET
    /v1/messages/123456?revision=2&sub.subfield=foo` | `GetMessage(message_id:
    "123456" revision: 2 sub: SubMessage(subfield: "foo"))` Note that fields
    which are mapped to HTTP parameters must have a primitive type or a
    repeated primitive type. Message types are not allowed. In the case of a
    repeated type, the parameter can be repeated in the URL, as in
    `...?param=A&param=B`. For HTTP method kinds which allow a request body,
    the `body` field specifies the mapping. Consider a REST update method on
    the message resource collection:     service Messaging {       rpc
    UpdateMessage(UpdateMessageRequest) returns (Message) {         option
    (google.api.http) = {           put: "/v1/messages/{message_id}"
    body: "message"         };       }     }     message UpdateMessageRequest {
    string message_id = 1; // mapped to the URL       Message message = 2;   //
    mapped to the body     } The following HTTP JSON to RPC mapping is enabled,
    where the representation of the JSON in the request body is determined by
    protos JSON encoding: HTTP | RPC -----|----- `PUT /v1/messages/123456 {
    "text": "Hi!" }` | `UpdateMessage(message_id: "123456" message { text:
    "Hi!" })` The special name `*` can be used in the body mapping to define
    that every field not bound by the path template should be mapped to the
    request body.  This enables the following alternative definition of the
    update method:     service Messaging {       rpc UpdateMessage(Message)
    returns (Message) {         option (google.api.http) = {           put:
    "/v1/messages/{message_id}"           body: "*"         };       }     }
    message Message {       string message_id = 1;       string text = 2;     }
    The following HTTP JSON to RPC mapping is enabled: HTTP | RPC -----|-----
    `PUT /v1/messages/123456 { "text": "Hi!" }` | `UpdateMessage(message_id:
    "123456" text: "Hi!")` Note that when using `*` in the body mapping, it is
    not possible to have HTTP parameters, as all fields not bound by the path
    end in the body. This makes this option more rarely used in practice of
    defining REST APIs. The common usage of `*` is in custom methods which
    don't use the URL at all for transferring data. It is possible to define
    multiple HTTP methods for one RPC by using the `additional_bindings`
    option. Example:     service Messaging {       rpc
    GetMessage(GetMessageRequest) returns (Message) {         option
    (google.api.http) = {           get: "/v1/messages/{message_id}"
    additional_bindings {             get:
    "/v1/users/{user_id}/messages/{message_id}"           }         };       }
    }     message GetMessageRequest {       string message_id = 1;       string
    user_id = 2;     } This enables the following two alternative HTTP JSON to
    RPC mappings: HTTP | RPC -----|----- `GET /v1/messages/123456` |
    `GetMessage(message_id: "123456")` `GET /v1/users/me/messages/123456` |
    `GetMessage(user_id: "me" message_id: "123456")` # Rules for HTTP mapping
    The rules for mapping HTTP path, query parameters, and body fields to the
    request message are as follows: 1. The `body` field specifies either `*` or
    a field path, or is    omitted. If omitted, it indicates there is no HTTP
    request body. 2. Leaf fields (recursive expansion of nested messages in the
    request) can be classified into three types:     (a) Matched in the URL
    template.     (b) Covered by body (if body is `*`, everything except (a)
    fields;         else everything under the body field)     (c) All other
    fields. 3. URL query parameters found in the HTTP request are mapped to (c)
    fields. 4. Any body sent with an HTTP request can contain only (b) fields.
    The syntax of the path template is as follows:     Template = "/" Segments
    [ Verb ] ;     Segments = Segment { "/" Segment } ;     Segment  = "*" |
    "**" | LITERAL | Variable ;     Variable = "{" FieldPath [ "=" Segments ]
    "}" ;     FieldPath = IDENT { "." IDENT } ;     Verb     = ":" LITERAL ;
    The syntax `*` matches a single path segment. The syntax `**` matches zero
    or more path segments, which must be the last part of the path except the
    `Verb`. The syntax `LITERAL` matches literal text in the path. The syntax
    `Variable` matches part of the URL path as specified by its template. A
    variable template must not contain other variables. If a variable matches a
    single path segment, its template may be omitted, e.g. `{var}` is
    equivalent to `{var=*}`. If a variable contains exactly one path segment,
    such as `"{var}"` or `"{var=*}"`, when such a variable is expanded into a
    URL path, all characters except `[-_.~0-9a-zA-Z]` are percent-encoded. Such
    variables show up in the Discovery Document as `{var}`. If a variable
    contains one or more path segments, such as `"{var=foo/*}"` or
    `"{var=**}"`, when such a variable is expanded into a URL path, all
    characters except `[-_.~/0-9a-zA-Z]` are percent-encoded. Such variables
    show up in the Discovery Document as `{+var}`. NOTE: While the single
    segment variable matches the semantics of [RFC
    6570](https://tools.ietf.org/html/rfc6570) Section 3.2.2 Simple String
    Expansion, the multi segment variable **does not** match RFC 6570 Reserved
    Expansion. The reason is that the Reserved Expansion does not expand
    special characters like `?` and `#`, which would lead to invalid URLs.
    NOTE: the field paths in variables and in the `body` must not refer to
    repeated fields or map fields.
    """

    selector: str = betterproto.string_field(1)
    """
    Selects methods to which this rule applies. Refer to
    [selector][google.api.DocumentationRule.selector] for syntax details.
    """

    get: str = betterproto.string_field(2, group="pattern")
    """Used for listing and getting information about resources."""

    put: str = betterproto.string_field(3, group="pattern")
    """Used for updating a resource."""

    post: str = betterproto.string_field(4, group="pattern")
    """Used for creating a resource."""

    delete: str = betterproto.string_field(5, group="pattern")
    """Used for deleting a resource."""

    patch: str = betterproto.string_field(6, group="pattern")
    """Used for updating a resource."""

    custom: "CustomHttpPattern" = betterproto.message_field(8, group="pattern")
    """
    The custom pattern is used for specifying an HTTP method that is not
    included in the `pattern` field, such as HEAD, or "*" to leave the HTTP
    method unspecified for this rule. The wild-card rule is useful for services
    that provide content to Web (HTML) clients.
    """

    body: str = betterproto.string_field(7)
    """
    The name of the request field whose value is mapped to the HTTP body, or
    `*` for mapping all fields not captured by the path pattern to the HTTP
    body. NOTE: the referred field must not be a repeated field and must be
    present at the top-level of request message type.
    """

    response_body: str = betterproto.string_field(12)
    """
    Optional. The name of the response field whose value is mapped to the HTTP
    body of response. Other response fields are ignored. When not set, the
    response message will be used as HTTP body of response.
    """

    additional_bindings: List["HttpRule"] = betterproto.message_field(11)
    """
    Additional HTTP bindings for the selector. Nested bindings must not contain
    an `additional_bindings` field themselves (that is, the nesting may only be
    one level deep).
    """


@dataclass(eq=False, repr=False)
class CustomHttpPattern(betterproto.Message):
    """A custom pattern is used for defining custom HTTP verb."""

    kind: str = betterproto.string_field(1)
    """The name of this custom HTTP verb."""

    path: str = betterproto.string_field(2)
    """The path matched by this custom verb."""


@dataclass(eq=False, repr=False)
class HttpBody(betterproto.Message):
    """
    Message that represents an arbitrary HTTP body. It should only be used for
    payload formats that can't be represented as JSON, such as raw binary or an
    HTML page. This message can be used both in streaming and non-streaming API
    methods in the request as well as the response. It can be used as a top-
    level request field, which is convenient if one wants to extract parameters
    from either the URL or HTTP template into the request fields and also want
    access to the raw HTTP body. Example:     message GetResourceRequest {
    // A unique request id.       string request_id = 1;       // The raw HTTP
    body is bound to this field.       google.api.HttpBody http_body = 2;     }
    service ResourceService {       rpc GetResource(GetResourceRequest) returns
    (google.api.HttpBody);       rpc UpdateResource(google.api.HttpBody)
    returns       (google.protobuf.Empty);     } Example with streaming
    methods:     service CaldavService {       rpc GetCalendar(stream
    google.api.HttpBody)         returns (stream google.api.HttpBody);
    rpc UpdateCalendar(stream google.api.HttpBody)         returns (stream
    google.api.HttpBody);     } Use of this type only changes how the request
    and response bodies are handled, all other features will continue to work
    unchanged.
    """

    content_type: str = betterproto.string_field(1)
    """
    The HTTP Content-Type header value specifying the content type of the body.
    """

    data: bytes = betterproto.bytes_field(2)
    """The HTTP request/response body as raw binary."""

    extensions: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(3)
    """
    Application specific response metadata. Must be set in the first response
    for streaming APIs.
    """



from dataclasses import dataclass

from ....base import BaseMessageWrapper
from .....proto.cosmos.base import v1beta1 as __base_v1_beta1__
from .....proto.cosmos.base.query import v1beta1 as __base_query_v1_beta1__
from .....proto.cosmos.staking.v1beta1 import MsgBeginRedelegate as MsgBeginRedelegateProto
from .....proto.cosmos.staking.v1beta1 import MsgDelegate as MsgDelegateProto
from .....proto.cosmos.staking.v1beta1 import MsgUndelegate as MsgUndelegateProto
from .....proto.tendermint import types as ___tendermint_types__

assert __base_v1_beta1__
assert __base_query_v1_beta1__
assert ___tendermint_types__


class MsgBeginRedelegate(BaseMessageWrapper, MsgBeginRedelegateProto):
    @property
    def type_url(self):
        return "/cosmos.staking.v1beta1.MsgBeginRedelegate"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgBeginRedelegate"


class MsgDelegate(BaseMessageWrapper, MsgDelegateProto):
    @property
    def type_url(self):
        return "/cosmos.staking.v1beta1.MsgDelegate"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgDelegate"


class MsgUndelegate(BaseMessageWrapper, MsgUndelegateProto):
    @property
    def type_url(self):
        return "/cosmos.staking.v1beta1.MsgUndelegate"

    @property
    def legacy_url(self):
        return "cosmos-sdk/MsgUndelegate"



from dataclasses import dataclass, fields
from typing import Optional, List

from ledgerblue.Dongle import Dongle
from ledgerblue.comm import getDongle
from ledgerblue.commException import CommException

from .exceptions import CosmosAppError
from .utils import get_bip32_byte, split_packet


@dataclass
class CosmosAppCommand:
    CLA: bytes
    INS: bytes
    P1: bytes
    P2: bytes
    L: Optional[bytes] = None
    data: Optional[bytes] = None

    def __post_init__(self):
        self.set_l()

    def set_l(self):
        if self.data is None:
            self.L = b"\x00"
        else:
            self.L = len(self.data).to_bytes(1, "little")

    def get_message(self):
        message = b""
        for field in fields(self.__class__):
            field_value = getattr(self, field.name)
            if field_value is not None:
                message += field_value
        return message


class CosmosAppResults:
    def __iter__(self):
        for var in self.__slots__:
            yield getattr(self, var)


class AppVersion(CosmosAppResults):
    __slots__ = ("cla", "major", "minor", "patch")

    def __init__(self, cla: int, major: int, minor: int, patch: int):
        self.cla = cla
        self.major = major
        self.minor = minor
        self.patch = patch

    def __repr__(self):
        return "AppVersion(cla={}, major={}, minor={}, patch={})".format(*self.__iter__())


class SepcAddr(CosmosAppResults):
    __slots__ = ("public_key", "address")

    def __init__(self, public_key: bytes, address: bytes):
        self.public_key = public_key
        self.address = address

    def __repr__(self):
        return "SepcAddr(public_key={}, address={})".format(*self.__iter__())


class CosmosApp:
    """Class for interacting with the CosmosApp on Ledger.

    Attributes:
        dongle: The dongle connection.
        derivation_path: The derivation path of the account to interact with.
    """

    def __init__(self, derivation_path: List[int], *, dongle: Optional[Dongle] = None):
        self.dongle = getDongle() if dongle is None else dongle
        self.derivation_path = derivation_path

    def __del__(self):
        self.disconnect()

    def disconnect(self) -> None:
        """Disconnects the ledger connection."""

        self.dongle.close()

    def get_version(self) -> AppVersion:
        """Gets the app version.

        Returns:
            The connected ledger's app version.

        Raises:
            CosmosAppError: Error returned from the ledger.
        """

        command = CosmosAppCommand(
            CLA=b"\x55",
            INS=b"\x00",
            P1=b"\x00",
            P2=b"\x00",
        )

        try:
            resp = self.dongle.exchange(command.get_message())
        except CommException as e:
            raise CosmosAppError(hex(e.sw))
        return AppVersion(*(resp[i] for i in range(4)))

    def ins_get_addr_secp256k1(self, hrp: str, req_confirm: bool = True) -> SepcAddr:
        """Gets the address using secp256k1.

        Args:
            hrp: Human-readable part of a Bech32 string.
            req_confirm: If confirmation is required on ledger before returning a result.

        Returns:
            The account address.

        Raises:
            CosmosAppError: Error returned from the ledger.
        """

        try:
            bip32_byte = get_bip32_byte(self.derivation_path, 3)
        except Exception as e:
            raise e

        command = CosmosAppCommand(
            CLA=b"\x55",
            INS=b"\x04",
            P1=req_confirm.to_bytes(1, "little"),
            P2=b"\x00",
            data=len(hrp).to_bytes(1, "little") + hrp.encode() + bip32_byte,
        )

        try:
            resp = self.dongle.exchange(command.get_message())
        except CommException as e:
            raise CosmosAppError(hex(e.sw))
        return SepcAddr(resp[:33], resp[33:])

    def sign_secp256k1(self, msg: bytes) -> bytearray:
        """Signs a given message using a private key generated on curve secp256k1.

        Args:
            msg: Message to sign.

        Returns:
            Signed message.

        Raises:
            CosmosAppError: Error returned from the ledger.
        """

        command = CosmosAppCommand(
            CLA=b"\x55",
            INS=b"\x02",
            P1=b"\x00",
            P2=b"\x00",
            data=get_bip32_byte(self.derivation_path, 3),
        )
        try:
            resp = self.dongle.exchange(command.get_message())
        except CommException as e:
            raise CosmosAppError(hex(e.sw))

        packets = split_packet(msg)
        for i, packet in enumerate(packets):
            if i == len(packets) - 1:
                payload_desc = b"\x02"
            else:
                payload_desc = b"\x01"
            command = CosmosAppCommand(
                CLA=b"\x55",
                INS=b"\x02",
                P1=payload_desc,
                P2=b"\x00",
                data=packet,
            )
            try:
                resp = self.dongle.exchange(command.get_message())
            except CommException as e:
                raise CosmosAppError(hex(e.sw))
        return resp



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/params/v1beta1/params.proto, cosmos/params/v1beta1/query.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class ParameterChangeProposal(betterproto.Message):
    """
    ParameterChangeProposal defines a proposal to change one or more
    parameters.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    changes: List["ParamChange"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ParamChange(betterproto.Message):
    """
    ParamChange defines an individual parameter change, for use in
    ParameterChangeProposal.
    """

    subspace: str = betterproto.string_field(1)
    key: str = betterproto.string_field(2)
    value: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """QueryParamsRequest is request type for the Query/Params RPC method."""

    subspace: str = betterproto.string_field(1)
    """subspace defines the module to query the parameter for."""

    key: str = betterproto.string_field(2)
    """key defines the key of the parameter in the subspace."""


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is response type for the Query/Params RPC method.
    """

    param: "ParamChange" = betterproto.message_field(1)
    """param defines the queried parameter."""


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.params.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.params.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/snapshots/v1beta1/snapshot.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class Snapshot(betterproto.Message):
    """Snapshot contains Tendermint state sync snapshot info."""

    height: int = betterproto.uint64_field(1)
    format: int = betterproto.uint32_field(2)
    chunks: int = betterproto.uint32_field(3)
    hash: bytes = betterproto.bytes_field(4)
    metadata: "Metadata" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class Metadata(betterproto.Message):
    """Metadata contains SDK-specific snapshot metadata."""

    chunk_hashes: List[bytes] = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class SnapshotItem(betterproto.Message):
    """SnapshotItem is an item contained in a rootmulti.Store snapshot."""

    store: "SnapshotStoreItem" = betterproto.message_field(1, group="item")
    iavl: "SnapshotIavlItem" = betterproto.message_field(2, group="item")
    extension: "SnapshotExtensionMeta" = betterproto.message_field(3, group="item")
    extension_payload: "SnapshotExtensionPayload" = betterproto.message_field(4, group="item")


@dataclass(eq=False, repr=False)
class SnapshotStoreItem(betterproto.Message):
    """SnapshotStoreItem contains metadata about a snapshotted store."""

    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class SnapshotIavlItem(betterproto.Message):
    """SnapshotIAVLItem is an exported IAVL node."""

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    version: int = betterproto.int64_field(3)
    """version is block height"""

    height: int = betterproto.int32_field(4)
    """height is depth of the tree."""


@dataclass(eq=False, repr=False)
class SnapshotExtensionMeta(betterproto.Message):
    """
    SnapshotExtensionMeta contains metadata about an external snapshotter.
    """

    name: str = betterproto.string_field(1)
    format: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class SnapshotExtensionPayload(betterproto.Message):
    """
    SnapshotExtensionPayload contains payloads of an external snapshotter.
    """

    payload: bytes = betterproto.bytes_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/mint/v1beta1/genesis.proto, cosmos/mint/v1beta1/mint.proto, cosmos/mint/v1beta1/query.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Minter(betterproto.Message):
    """Minter represents the minting state."""

    inflation: str = betterproto.string_field(1)
    """current annual inflation rate"""

    annual_provisions: str = betterproto.string_field(2)
    """current annual expected provisions"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params holds parameters for the mint module."""

    mint_denom: str = betterproto.string_field(1)
    """type of coin to mint"""

    inflation_rate_change: str = betterproto.string_field(2)
    """maximum annual change in inflation rate"""

    inflation_max: str = betterproto.string_field(3)
    """maximum inflation rate"""

    inflation_min: str = betterproto.string_field(4)
    """minimum inflation rate"""

    goal_bonded: str = betterproto.string_field(5)
    """goal of percent bonded atoms"""

    blocks_per_year: int = betterproto.uint64_field(6)
    """expected blocks per year"""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: "Params" = betterproto.message_field(1)
    """params defines the parameters of the module."""


@dataclass(eq=False, repr=False)
class QueryInflationRequest(betterproto.Message):
    """
    QueryInflationRequest is the request type for the Query/Inflation RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryInflationResponse(betterproto.Message):
    """
    QueryInflationResponse is the response type for the Query/Inflation RPC
    method.
    """

    inflation: bytes = betterproto.bytes_field(1)
    """inflation is the current minting inflation value."""


@dataclass(eq=False, repr=False)
class QueryAnnualProvisionsRequest(betterproto.Message):
    """
    QueryAnnualProvisionsRequest is the request type for the
    Query/AnnualProvisions RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryAnnualProvisionsResponse(betterproto.Message):
    """
    QueryAnnualProvisionsResponse is the response type for the
    Query/AnnualProvisions RPC method.
    """

    annual_provisions: bytes = betterproto.bytes_field(1)
    """annual_provisions is the current minting annual provisions value."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the mint module's genesis state."""

    minter: "Minter" = betterproto.message_field(1)
    """minter is a space for holding current inflation information."""

    params: "Params" = betterproto.message_field(2)
    """params defines all the paramaters of the module."""


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.mint.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def inflation(
        self,
        query_inflation_request: "QueryInflationRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryInflationResponse":
        return await self._unary_unary(
            "/cosmos.mint.v1beta1.Query/Inflation",
            query_inflation_request,
            QueryInflationResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def annual_provisions(
        self,
        query_annual_provisions_request: "QueryAnnualProvisionsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAnnualProvisionsResponse":
        return await self._unary_unary(
            "/cosmos.mint.v1beta1.Query/AnnualProvisions",
            query_annual_provisions_request,
            QueryAnnualProvisionsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def inflation(self, query_inflation_request: "QueryInflationRequest") -> "QueryInflationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def annual_provisions(
        self, query_annual_provisions_request: "QueryAnnualProvisionsRequest"
    ) -> "QueryAnnualProvisionsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_inflation(
        self,
        stream: "grpclib.server.Stream[QueryInflationRequest, QueryInflationResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.inflation(request)
        await stream.send_message(response)

    async def __rpc_annual_provisions(
        self,
        stream: "grpclib.server.Stream[QueryAnnualProvisionsRequest, QueryAnnualProvisionsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.annual_provisions(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.mint.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/cosmos.mint.v1beta1.Query/Inflation": grpclib.const.Handler(
                self.__rpc_inflation,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryInflationRequest,
                QueryInflationResponse,
            ),
            "/cosmos.mint.v1beta1.Query/AnnualProvisions": grpclib.const.Handler(
                self.__rpc_annual_provisions,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAnnualProvisionsRequest,
                QueryAnnualProvisionsResponse,
            ),
        }
